#include "core.h"

Core::Core() {}

void Core::initCore()
{
    //初始化各部分
    // model/view
    m_model = new QStandardItemModel();
    m_collectModel = new QStandardItemModel();
    // dbus接口用-暂时不用
    //    m_canProcess = new QTimer(this);
    //    m_canProcess->setSingleShot(true);
    //网络模块
    m_network = new Network;
    // ip定位模块
    m_ipLocation = new Iplocation;
    //数据解析
    m_dataParser = new DataParser;
    //数据请求
    m_dataRequest = new DataRequest;
    //搜索模块
    m_searchMarch = new SearchMarch;
    //定时器-30分钟刷新
    m_refreshweather = new QTimer(this);              //定时更新主界面天气
    m_refreshweather->setTimerType(Qt::PreciseTimer); //精密计时,保持毫秒精度

    //自定义类型数据注册
    qRegisterMetaType<LifeStyle>("LifeStyle");
    qRegisterMetaType<CurrentWeather>("CurrentWeather");
    qRegisterMetaType<ForecastWeather>("ForecastWeather");
    qRegisterMetaType<LocationData>("LocationData");
    qRegisterMetaType<ItemData>("ItemData");
    qRegisterMetaType<QList<QPersistentModelIndex>>("QList<QPersistentModelIndex>");
    qRegisterMetaType<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");

    //初始化信号槽
    initConnect();
    //检查网络
    m_network->checkNetWork();
}
//信号槽
void Core::initConnect()
{
    connect(m_network, &Network::effectiveNetwork, this, &Core::networkState); //检查网络状态
    connect(m_network, &Network::haveNet, this, &Core::refreshCity);           //网络从无到有，开始重新请求
    connect(m_dataRequest, &DataRequest::sendData, m_dataParser, &DataParser::dataReply);         //数据reply
    connect(m_dataRequest, &DataRequest::sendCollData, m_dataParser, &DataParser::dataReplyColl); //收藏城市数据reply
    connect(m_dataParser, &DataParser::showCurrent, this, &Core::showCurrent); //发送信号，显示生活指数信息
    connect(m_dataParser, &DataParser::showForcast, this, &Core::showForcast); //发送信号，显示当前城市信息
    connect(m_dataParser, &DataParser::showLifeStyle, this, &Core::showLifeStyle); //发送信号，显示城市预测信息
    connect(m_dataParser, &DataParser::showCollectCode, this, &Core::updateCollect); //更新收藏城市信息
    connect(m_dataParser, &DataParser::showHourly, this, &Core::showHourly);         //增加逐小时临时调试接口
    connect(m_refreshweather, &QTimer::timeout, this, &Core::refreshCity); //设置定时器，每隔半小时刷新
    m_refreshweather->start((30 * 60) * 1000);                             //定时器开启-30mim
}
//城市切换--收藏，搜索，热门
void Core::changeCity(QString cityId)
{
    if (cityId == "") {
        ErrorReact::getInstance()->collectAddError("切换城市为空");
        return;
    }
    //获取城市数据
    m_dataRequest->dataRequest(cityId);
}
//从搜索列表改变当前城市
void Core::changeCityFromSearch(QModelIndex index)
{
    ItemData itemda = m_model->itemFromIndex(index)->data(Qt::UserRole).value<ItemData>();
    if (itemda.cityId.isEmpty() || itemda.cityName.isEmpty()) {
        return;
    }
    changeCity(itemda.cityId);
}
//加载收藏城市
void Core::loadCollectCity()
{
    QString collectCity = CoreVar::getSettings("collect-city").toString();
    QStringList listCollectCity = collectCity.split(",");
    m_fullCityList = listCollectCity;
    m_open = false;
    //判断配置文件有内容，发信号给前端改变控件位置
    if (collectCity != "") {
        emit collectShow();
    }
    for (int i = 0; i < listCollectCity.length(); i++) {
        if (listCollectCity.at(i) != "") {
            m_updateList = false; //--备用
            m_itemIndex = i;
            m_searchResultList = m_searchMarch->exactMatchCity(listCollectCity.at(i));
            LocationData m_locationdata = (LocationData)m_searchResultList.at(0);
            initCollectModel(listCollectCity.at(i), m_locationdata.city);
        }
    }
    //暂时先一个一个访问
    requestDataColl(collectCity);
}
//显示在屏幕中间
void Core::showInCenter()
{
    emit showCenter();
}
//添加收藏城市
void Core::addCollectCity(QModelIndex index)
{
    //获得点击的城市的cityId--不能用cityName来判断，会重名。
    QStandardItem *item = m_model->itemFromIndex(index);
    ItemData itemdata = item->data(Qt::UserRole).value<ItemData>();

    if (itemdata.cityId.isEmpty() || itemdata.cityName.isEmpty() || itemdata.cityId == "") {
        ErrorReact::getInstance()->collectAddError("添加收藏城市异常，数据为空");
        return;
    }
    //遍历收藏城市列表，不可重复添加
    if (!m_fullCityList.isEmpty()) {
        for (QStringList::iterator it = m_fullCityList.begin(); it != m_fullCityList.end(); ++it) {
            if (*it == itemdata.cityId) { //若收藏城市列表中已有需要添加的城市，则返回
                ErrorReact::getInstance()->collectAddError("收藏城市列表中已有需要添加的城市");
                return;
            }
        }
    }
    //设置将城市加入配置文件
    QString fullCityID = itemdata.cityId;
    m_fullCityList.append(fullCityID);
    //更新配置文件
    //重新将收藏城市列表写入配置文件
    setCollectGsettings();
    //更改状态
    m_open = true;
    m_updateList = true;
    //设置item，model--用于界面显示
    initCollectModel(itemdata.cityId, itemdata.cityName);
}
//从收藏城市切换城市
void Core::changeCityFromCollect(QModelIndex index)
{
    CollectItemData itemda = m_collectModel->itemFromIndex(index)->data(Qt::UserRole).value<CollectItemData>();
    if (itemda.cityId.isEmpty() || itemda.cityName.isEmpty()) {
        ErrorReact::getInstance()->collectAddError("收藏城市切换主界面城市错误");
        return;
    }
    //切换城市
    changeCity(itemda.cityId);
}
//删除收藏列表中的城市
void Core::delCollectCity(QModelIndex index)
{
    //遍历列表，拿到需要删除的item对应的id值 -更新配置文件
    QString sign;
    if (m_collectModel != nullptr) {
        //获取收藏列表中的cityId
        sign = m_collectModel->item(index.row(), index.column())->data(Qt::UserRole).value<CollectItemData>().cityId;
    }
    //删除选中项--设置一个list存其余各项
    QList<QStandardItem *> itemList;
    //遍历当前整个model
    for (int i = 0; i < m_collectModel->rowCount(); i++) {
        for (int j = 0; j < m_collectModel->columnCount(); j++) {
            //判断为删除项，跳出此次，继续遍历
            if (index.row() == i && index.column() == j) {
                //判断到最后一个item的时候，可以直接跳出循环。
                if (m_collectModel->item(i, j)->data(Qt::UserRole).value<CollectItemData>().cityId
                    == m_fullCityList.at(m_fullCityList.length() - 1)) {
                    break;
                } else {
                    continue;
                }
            }
            //遍历到最后一行时，对于空item，continue--不然会发生闪退
            if (i == m_collectModel->rowCount() - 1
                && i * m_collectModel->columnCount() + (j + 1) > m_fullCityList.length() - 1) {
                continue;
            }
            //将有效项加入qlist
            itemList.append(m_collectModel->item(i, j)->clone());
        }
    }

    //清空model，重排整个model
    m_collectModel->clear();
    //遍历qlist，将有效item重新排列
    for (int i = 0; i < itemList.length(); i++) {
        m_collectModel->setItem(i / 3, i % 3, itemList.at(i));
    }

    //更新配置文件
    if (!m_fullCityList.empty()) {
        int cityIndex = m_fullCityList.indexOf(sign);
        m_fullCityList.removeAt(cityIndex);
        //判断配置文件为空，发送信号给前端
        if (m_fullCityList.length() == 1) {
            emit collectNull();
        }
        setCollectGsettings();
    }
}

void Core::allowLocationYes()
{
    CoreVar::setSettings("allow-locate", "1"); //改变定位为允许定位
    location();
}

void Core::allowLocationNo()
{
    CoreVar::setSettings("allow-locate", "0"); //不允许定位
}

//定位
void Core::location()
{
    //判断是否允许定位----默认不允许定位
    if ("1" == CoreVar::getSettings("allow-locate").toString()) {
        //定位--地区名
        QString cityName = m_ipLocation->gotoLocation();
        //发现ipip定位偶尔会出现返回乱码--包含http字段
        if (cityName.contains("http")) {
            ErrorReact::getInstance()->geoFail(tr("Seek failed!"));
            //           cityName = m_ipLocation->gotoLocation();
        }
        //定位失败
        if (cityName == "" || cityName.contains("http")) {
            ErrorReact::getInstance()->geoFail(tr("Seek failed!"));
            m_currentCityId = "";
            refreshCity();
            return;
        }
        //搜索本地词库--地区id
        QString cityId = m_searchMarch->setAutomaticCity(cityName);
        m_dataRequest->dataRequest(cityId);
        m_currentCityId = cityId;
    } else if ("0" == CoreVar::getSettings("allow-locate").toString()) {
        //不允许定位时
        m_dataRequest->dataRequest(CoreVar::DEFAULT_CITYID);
        m_currentCityId = CoreVar::DEFAULT_CITYID;
    }
}

//搜索城市
void Core::searchCity(QString inputText)
{
    //清空模型-清空上次搜索结果
    m_model->clear();
    //去搜索模块获得搜索结果list
    m_searchResultList = m_searchMarch->exactMatchCity(inputText);
    if (inputText == "") {
        //输入为空，发信号给前端进行处理
        emit searchUiShow(false);
    } else if (m_searchResultList.isEmpty()) {
        //搜索为空，发信号给前端进行处理
        emit searchNull();
    } else {
        emit searchUiShow(true);
        //设置搜索模型
        setModel();
    }
}
//请求收藏数据
void Core::requestDataColl(QString cityIdList)
{
    m_dataRequest->dataRequestCollect(cityIdList);
}
//刷新城市
void Core::refreshCity()
{
    //拿数据
    //    m_network->checkNetWork();
    if (m_currentCityId == "") {
        QString currentInfo = CoreVar::getSettings("weather").toString();
        QStringList currentInfoList = currentInfo.split(",");
        if (currentInfoList.length() >= 8) {
            m_currentCityId = currentInfoList.at(1);
        } else {
            m_currentCityId = CoreVar::DEFAULT_CITYID;
        }
    }
    m_dataRequest->dataRequest(m_currentCityId);
}
//搜索model
QStandardItemModel *Core::getCityModel()
{
    return m_model;
}
//收藏model
QStandardItemModel *Core::getCollectModel()
{
    return m_collectModel;
}
//设置搜索model数据
void Core::setModel()
{
    //遍历整个搜索结果列表
    for (int i = 0; i < m_searchResultList.size(); ++i) {
        LocationData m_locationdata = (LocationData)m_searchResultList.at(i);
        //设置模型item-构造搜索model
        initModel(m_locationdata, i);
    }
}
//构造搜索model
void Core::initModel(LocationData m_locationdata, int i)
{
    // model数据
    ItemData m_itemData;
    //城市id
    m_itemData.cityId = m_locationdata.id;
    //城市名
    m_itemData.cityName = m_locationdata.city;
    //初始化item
    QStandardItem *item = new QStandardItem;
    //设置不可拖拽
    item->setDragEnabled(false);
    //整体存取
    item->setData(QVariant::fromValue(m_itemData), Qt::UserRole);
    //插入model
    m_model->setItem(i / 3, i % 3, item);
}

//构造搜索model
void Core::initCollectModel(QString cityId, QString cityName)
{
    //判断是从搜索界面加入收藏城市，需要知道当前共有多少个城市
    if (m_open) {
        m_itemIndex = m_fullCityList.length() - 2;
    }
    // model数据
    CollectItemData m_collectItemData;
    //城市id
    m_collectItemData.cityId = cityId;
    //城市名
    m_collectItemData.cityName = cityName;
    //设置item
    QStandardItem *item = new QStandardItem;
    //设置item不可拖拽
    item->setDragEnabled(false);
    //整体存取
    item->setData(QVariant::fromValue(m_collectItemData), Qt::UserRole);
    item->setToolTip(cityName);

    //设置model的行列
    m_collectModel->setItem(m_itemIndex / 3, m_itemIndex % 3, item);
    //请求数据---获取温度和阴晴
    if (m_updateList) {
        requestDataColl(m_collectItemData.cityId);
    }
}


//初始化dbus
QString Core::initDbus(const QStringList &arguments)
{
    //    m_dbus = new Dbus;-------暂不需要
}

void Core::processingCommand(const QStringList &cmd)
{
    if (coreOperateTooOften()) {
        return;
    }
    if (cmd[0] == "-change") {
        //调用函数实现功能
        return;
    }
}

QString Core::processingApi(const QStringList &cmd) {}

void Core::progremExit()
{
    this->deleteLater();
    emit coreProgremExit();
}

bool Core::apiFunction() {}

//配置文件存收藏城市
void Core::setCollectGsettings()
{
    QString newStrCityId = "";
    foreach (QString str, m_fullCityList) {
        if (str != "") {
            newStrCityId.append(str);
            newStrCityId.append(",");
        }
    }
    CoreVar::setSettings("collect-city", newStrCityId); //将更新后的列表写入配置文件
}

void Core::networkState(bool isOnline)
{
    //启动
    if (isOnline) {
        //开始定位
        location();
    } else {
        //无网络或其他错误
        return;
    }
}

//更新收藏城市列表
void Core::updateCollect(QList<CollectItemData> collectItemList)
{
    if (m_updateList) {
        //遍历当前整个model
        for (int i = 0; i < m_collectModel->rowCount(); i++) {
            for (int j = 0; j < m_collectModel->columnCount(); j++) {
                //遍历到最后一行时，对于空item，跳出--不然会发生闪退
                if (i == m_collectModel->rowCount() - 1
                    && i * m_collectModel->columnCount() + (j + 1) > m_fullCityList.length() - 1) {
                    break;
                } else {
                    QStandardItem *item = m_collectModel->item(i, j);
                    if (item->data(Qt::UserRole).value<CollectItemData>().cityId == collectItemList.at(0).cityId) {
                        m_collectModel->item(i, j)->setData(QVariant::fromValue(collectItemList.at(0)), Qt::UserRole);
                        //备用--收藏列表最后一个加载完成--以后可能会需要这个完成的信号
                        //                   if (i == m_collectModel->rowCount() - 1 && i *m_collectModel->columnCount()
                        //                   + (j+1) == m_fullCityList.length() - 1) {
                        //                       qDebug()<<"最后一个加载完成";
                        //                       emit updateFinish();
                        return;
                    }
                }
            }
        }
    }
    //遍历当前整个model
    for (int i = 0; i < m_collectModel->rowCount(); i++) {
        for (int j = 0; j < m_collectModel->columnCount(); j++) {
            //遍历到最后一行时，对于空item，跳出--不然会发生闪退
            if (i == m_collectModel->rowCount() - 1
                && i * m_collectModel->columnCount() + (j + 1) > m_fullCityList.length() - 1) {
                break;
            } else {
                QStandardItem *item = m_collectModel->item(i, j);
                for (int k = 0; k < collectItemList.length(); k++) {
                    if (item->data(Qt::UserRole).value<CollectItemData>().cityId == collectItemList.at(k).cityId) {
                        m_collectModel->item(i, j)->setData(QVariant::fromValue(collectItemList.at(k)), Qt::UserRole);
                    }
                }
            }
        }
    }
}

bool Core::coreOperateTooOften()
{
    if (m_canProcess->isActive()) {
        return true;
    }
    m_canProcess->start(CoreVar::REFRESH_RATE); //刷新间隔
    return false;
}
