#include "controller/core/viewcontrol.h"
#include <ukui-log4qt.h>
#include <QtSingleApplication>
#include <QApplication>
#include <QObject>
#include <signal.h>
#include <QDBusInterface>
//单例
bool onlyOne(QtSingleApplication &a)
{
    if (a.isRunning()) {
        qInfo() << "给首个对象发送信号："
                << a.sendMessage(QApplication::arguments().length() > 1 ? QApplication::arguments().at(1)
                                                                        : a.applicationFilePath());
        qInfo() << "Can't lock single file, kylin-weather is already running!";
        return true;
    }
    return false;
}

//初始化核心，初始化前端，启动时是否展示界面
void controlView(QtSingleApplication &a, const QStringList &args, bool isStart)
{
    //实例化viewcontrol
    viewControl view;
    view.initCore(args, isStart);
    //解决带参启动，无actionWindow问题
    QObject::connect(&view, SIGNAL(setActiveWidow(QWidget *)), &a, SLOT(setActivationWindow(QWidget *)));
    //激活窗口
    a.setActivationWindow(view.activeWindow());
}
int main(int argc, char *argv[])
{
    //添加4K 屏幕支持。
#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
#else
    // do not support 4K.
#endif
    //添加9A0基于qt5.15 屏幕支持。
#if (QT_VERSION >= QT_VERSION_CHECK(5, 12, 0))
    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
#endif
#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
    QApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
#endif

    QtSingleApplication a(argc, argv);
    /*方法一：直接在QApplication下设置窗口图标
    这样设置的图标会自动跟随图标主题变化*/
    a.setWindowIcon(QIcon::fromTheme("kylin-weather"));

    //添加log日志
    initUkuiLog4qt("kylin-weather");
    //检查单例
    if (onlyOne(a))
        return 0;
    //翻译文件加载
    QTranslator app_trans;
    QTranslator qt_trans;
    QString locale = QLocale::system().name();
    QString trans_path;
    if (QDir("/usr/share/kylin-weather/translations").exists()) {
        trans_path = "/usr/share/kylin-weather/translations";
    } else {
        trans_path = qApp->applicationDirPath() + "/translations";
    }
    QString qt_trans_path;
    qt_trans_path = QLibraryInfo::location(QLibraryInfo::TranslationsPath); // /usr/share/qt5/translations
    if (!app_trans.load("kylin-weather_" + locale + ".qm", trans_path)) {
        qInfo() << "Load translation file："
                << "kylin-weather_" + locale + ".qm from" << trans_path << "failed!";
    } else {
        a.installTranslator(&app_trans);
    }
    if (!qt_trans.load("qt_" + locale + ".qm", qt_trans_path)) {
        qInfo() << "Load translation file："
                << "qt_" + locale + ".qm from" << qt_trans_path << "failed!";
    } else {
        a.installTranslator(&qt_trans);
    }
    //初始化后端前端，判断启动方式，是否带参
    controlView(a, a.arguments(), !(argc == 2 && QLatin1String(argv[1]) == "showmainwindow"));

    return a.exec();
}
