#include "searchmarch.h"
#include <QDebug>
#include <QFile>

SearchThread::SearchThread(SearchMarch *parent) :
    QThread(parent)
{

}

void SearchThread::run()
{
    SearchMarch *m_worker = qobject_cast<SearchMarch*>(parent());
    if (!m_worker) {
        return;
    }
    //CN101250101,changsha,长沙,CN,China,中国,hunan,湖南,changsha,长沙,28.19409,112.98228,"430101,430100,430000",

    //本地词表--存储各地区的中英文名称，对应的cityId，经纬度等信息
    //在开始时，将本地词表中的所有的地区的信息m_locatonList列表中，元素为LocationData结构体
    QFile file(":/data/data/china-city-list.csv");
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        //读取行
        QString line = file.readLine();
        line = line.replace("\n", "");
        //去掉首行，没有数据的，遍历整个词表csv，将其存储在m_locatonList列表中
        while (!line.isEmpty()) {
            QStringList resultList = line.split(",");
            if (resultList.length() < 10) {
                line = file.readLine();
                line = line.replace("\n", "");
                continue;
            }

            QString id = resultList.at(0);
            if (!id.startsWith("CN")) {
                line = file.readLine();
                line = line.replace("\n", "");
                continue;
            }

            LocationData data;
            data.id = id.remove(0, 2);//remove "CN"
            data.city_en = resultList.at(1);
            data.city = resultList.at(2);
            data.country_en = resultList.at(4);
            data.country = resultList.at(5);
            data.province_en = resultList.at(6);
            data.province = resultList.at(7);
            data.admin_district_en = resultList.at(8);
            data.admin_district = resultList.at(9);
            data.shorthand = resultList.at(10);

            m_worker->m_locatonList << data; //将数据存入m_locatonList中

            line = file.readLine();
            line = line.replace("\n", "");
        }
        file.close();
    }
}


SearchMarch::SearchMarch(QObject *parent) : QObject(parent)
{
    m_workerThread = new SearchThread(this);
    m_workerThread->start();
}

SearchMarch::~SearchMarch()
{
    m_workerThread->deleteLater();
}
//返回搜索结果
QList<LocationData> SearchMarch::exactMatchCity(const QString &inputText) const
{
    QList<LocationData> searchedList;
    //搜索匹配-遍历m_locatonList
    for (const LocationData line : m_locatonList) {
        if (line.shorthand == inputText || line.id == inputText ||                                    //拼音和ID
            line.province == inputText ||line.province_en == inputText ||                             //省
            line.city.contains(inputText) || line.city_en.contains(inputText)                         //县
            ) {
            //保证完全匹配的字符串放在显示的第一个
            if (inputText == line.city_en) {
                searchedList.insert(0,line);
            } else {
                searchedList.append(line); //将数据加入列表
            }
        }
    }
    return searchedList;
}
//自动定位-查找当前地区id
QString SearchMarch::setAutomaticCity(const QString &cityName)
{
    QString cityId;

    if (cityName.isEmpty()) {
        return cityId;
    }

    QFile file(":/data/data/china-city-list.csv");
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QString line = file.readLine();

        line = line.replace("\n", "");
        while (!line.isEmpty()) {
            QStringList resultList = line.split(",");
            if (resultList.length() < 10) {
                line = file.readLine();
                line = line.replace("\n", "");
                continue;
            }
            //("CN101340406", "yunlin", "云林", "CN", "China", "中国", "taiwan", "台湾", "taizhong", "台中", "yl", "23.718", "120.538", "710000")
            QString id = resultList.at(0);
            if (!id.startsWith("CN")) {
                line = file.readLine();
                line = line.replace("\n", "");
                continue;
            }
            //对比cityname和词表的city，拿id，去CN
            if (resultList.at(1).compare(cityName, Qt::CaseInsensitive) == 0 ||
                resultList.at(2).compare(cityName, Qt::CaseInsensitive) == 0 ||
                QString(resultList.at(2) + "市").compare(cityName, Qt::CaseInsensitive) == 0 ||
                QString(resultList.at(2) + "区").compare(cityName, Qt::CaseInsensitive) == 0 ||
                QString(resultList.at(2) + "县").compare(cityName, Qt::CaseInsensitive) == 0) {
                id.remove(0, 2);//remove "CN"
                cityId = id;
                break;
            }

            line = file.readLine();
            line = line.replace("\n", "");
        }
        file.close();
    }
    return cityId;
}
