#include "flu.h"
#include <QVBoxLayout>
#include <QGraphicsOpacityEffect>
#define UKUI_STYLE "org.ukui.style"
#define UKUI_FONT_SIZE "systemFontSize"
flu::flu(QWidget *parent) : QWidget(parent)
{
    m_fluLayout = new QVBoxLayout(this);

    m_fluIcon = new QLabel(this);
    m_str = new QLabel(this);
    m_degree = new QLabel(this);

    QPixmap flupixmap(":/res/contorl_icons/index_flu.png");
    m_fluIcon->setPixmap(flupixmap);

    m_str->setText(tr("Flu"));

    m_degree->setText("N/A");

    initLayout();
//    initQss();
}

void flu::setItemContent(QString text)
{
    m_degree->setText(text);
}

void flu::isFullScreen(int m_isFullScreen)
{
    m_judgeFullScreen = m_isFullScreen;
    initQss(14.0);
}

void flu::initLayout()
{
    m_fluLayout->addWidget(m_fluIcon);
    m_fluLayout->addWidget(m_str);
    m_fluLayout->addWidget(m_degree);
}

void flu::lable2SetFontSizeSlot(int size)
{
    //默认大小12px,换算成pt为9
    double lable2BaseFontSize = 11;//魔鬼数字，自行处理
    double nowFontSize = lable2BaseFontSize * double(size) / 11;//11为系统默认大小，魔鬼数字，自行处理
//    QFont font;
//    font.setPointSizeF(nowFontSize);
//    ui->label2->setFont(font);
    initQss(nowFontSize);
}

void flu::initFontSize()
{
    //只有非标准字号的控件才需要绑定
    connect(this,&flu::setFontSizeSignal,this,&flu::lable2SetFontSizeSlot);

    //判断gsettings是否存在
    if(QGSettings::isSchemaInstalled(UKUI_STYLE)) {
        m_fontSize = new QGSettings(UKUI_STYLE);
        //监听系统字号大小改变事件
        connect(m_fontSize,&QGSettings::changed,this, [=] (const QString &key) {
            if(key == UKUI_FONT_SIZE) {
                //获取字号的值
                int fontSizeKey = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
                //发送改变信号
                if (fontSizeKey > 0) {
                    emit setFontSizeSignal(fontSizeKey);
                }
            }
        });
    }
    //启动时设置字号
    int fontSizeKey = 11;//系统默认字号,魔鬼数字，自行处理
    if (m_fontSize != nullptr) {
        int fontSizeKeyTmp = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
        if (fontSizeKeyTmp > 0) {
            fontSizeKey = fontSizeKeyTmp;
        }
    }
    emit setFontSizeSignal(fontSizeKey);
}

void flu::initQss(double nowFontSize)
{
    m_fluIcon->setAlignment(Qt::AlignCenter);

    if(m_judgeFullScreen == 0){
        m_fluIcon->resize(30,30);
        QPixmap flupixmap(flupath);
        m_fluIcon->setPixmap(flupixmap);

        //    m_str->setStyleSheet("font:14px;color:white;");
        QFont m_font;
//        m_font.setPointSizeF(nowFontSize);
        m_str->setFont(m_font);
        m_str->setAlignment(Qt::AlignCenter);
        QPalette fluStrPalette = m_str->palette();
        fluStrPalette.setColor(QPalette::WindowText,Qt::white);
        m_str->setPalette(fluStrPalette);
        //    设置字体透明度
        QGraphicsOpacityEffect *fluOpacityEffect=new QGraphicsOpacityEffect;
        fluOpacityEffect->setOpacity(0.7);
        m_str->setGraphicsEffect(fluOpacityEffect);
        //    m_degree->setStyleSheet("font:14px;color:white;");
        m_degree->setAlignment(Qt::AlignCenter);
        QPalette fluDegreePalette = m_degree->palette();
        fluDegreePalette.setColor(QPalette::WindowText,Qt::white);
        m_degree->setPalette(fluDegreePalette);
        m_degree->setFont(m_font);
    }
    else {
        m_fluIcon->resize(60,60);
        QPixmap flupixmap(flupath);
        flupixmap = flupixmap.scaled(m_fluIcon->width(),m_fluIcon->height(),Qt::KeepAspectRatio, Qt::SmoothTransformation);
        m_fluIcon->setPixmap(flupixmap);

        QFont m_font;
        m_font.setPixelSize(28);
        m_str->setFont(m_font);
        m_degree->setFont(m_font);
    }
}
