#include "horscreen.h"
#include "title.h"
#include "city.h"
#include "perhour.h"
#include "weekweather.h"
#include "onedayweather.h"
#include "lifestyle.h"
#include "information.h"
#include <QVBoxLayout>
#include <QScrollArea>
#include <QScrollBar>
#include "daemondbus.h"
#include "view/horscreen/xatom-helper.h"
#include "global/corevar.h"
#include "view/horscreen/allowlocation.h"
horscreen *horscreen::mutual = nullptr;

horscreen::horscreen(QWidget *parent) : QWidget(parent)
{
    initInteraction();

    //主题框架
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);

    //    setWindowFlags(Qt::FramelessWindowHint);
    this->setFixedSize(900, 620);

    this->setWindowTitle(tr("Weather"));

    m_pixmap = QPixmap(":/res/background/100.png");

    mutual = this;
    m_label = new QLabel(this);
    m_label->move(0, 0);

    m_mainLayout = new QVBoxLayout(this);

    m_mainCityLayout = new QVBoxLayout();

    m_title = new title(this);

    m_empty = new Empty(this);
    m_city = new city(this);

    m_scrollarea = new QScrollArea(this);
    m_scrollarea->setFocusPolicy(Qt::NoFocus); //设置焦点类型

    //    m_scrollarea->setStyleSheet("QScrollArea{border:none;border-radius:4px;background:transparent;color:rgba(255,255,255,1);}");
    //    m_scrollarea->setStyleSheet("QScrollArea{background:transparent;}");
    m_scrollarea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_scrollarea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    m_scrollarea->verticalScrollBar()->setProperty("drawScrollBarGroove", false);
    // 设置scrollarea背景透明
    QPalette scrollarea = m_scrollarea->palette();
    scrollarea.setColor(QPalette::Background, QColor(255, 255, 255, 0));
    m_scrollarea->setPalette(scrollarea);
    m_scrollarea->setFrameShape(QFrame::NoFrame); // 设置滑动区无边框

    m_scrollwidget = new QWidget(m_scrollarea);
    //    m_scrollwidget->setStyleSheet("QWidget{background:transparent;color:rgba(255,255,255,1);}");
    //    m_scrollwidget->setAttribute(Qt::WA_TranslucentBackground);
    //    m_scrollwidget->setWindowFlag(Qt::FramelessWindowHint);

    m_scrollarea->setWidget(m_scrollwidget);
    //    m_scrollwidget->move(0, m_city->y() + m_city->height() +20);
    m_information = new information(m_scrollwidget);

    m_searchView = new SearchView(this);
    m_searchView->hide();
    m_searchView->move((this->width() - m_searchView->width()) / 2, (this->height() - m_searchView->height()) / 2);

    // 用户手册功能
    m_DaemonIpcDbus = new DaemonDbus();

    m_hintWidget = new PromptWidget(this);
    m_hintWidget->setVisible(false);

    //    m_AllowLocationwidget = new AllowLocation(this);
    //    m_AllowLocationwidget->resize(350,174);
    //    m_AllowLocationwidget->show();

    this->initConnect();
    initLayout();
}

void horscreen::changeBackgroundImage(CurrentWeather m_currentWeather)
{
    QString code = m_currentWeather.cond_code;
    QString path = ":/res/background/" + code + ".png";
    m_pixmap = QPixmap(path);
    m_pixmap = m_pixmap.scaled(m_label->width(), m_label->height(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
    m_label->setPixmap(m_pixmap);
}

void horscreen::initLayout()
{
    m_mainLayout->addWidget(m_title);
    m_mainCityLayout->addWidget(m_city);
    m_mainLayout->addLayout(m_mainCityLayout);
    //    m_mainLayout->addWidget(m_city);

    m_mainLayout->addWidget(m_scrollarea);
    m_scrollarea->setContentsMargins(0, 0, 0, 0);
    m_mainLayout->setContentsMargins(0, 0, 0, 0);
    this->setLayout(m_mainLayout);
}

void horscreen::changeHeight(double detio)
{
    //    m_city->setMinimumSize(this->width(),180*detio);
    if (detio > 1) {
        m_city->m_centrlWid->setFixedSize(1.2 * 360 * detio, 1.2 * 180 * detio);
    } else {
        //        m_city->m_centrlWid->setFixedSize(420*detio,180*detio);
        m_city->m_centrlWid->setFixedSize(455 * detio, 180 * detio);
    }
    m_scrollwidget->resize(this->width() - 35, (170 + 223 + 140) * detio);
    m_information->resize(this->width() - 35, m_scrollwidget->height());
    m_scrollarea->resize(this->width(), m_scrollwidget->height());
}

// void horscreen::mouseDoubleClickEvent(QMouseEvent *event)
//{
//    //判断左键双击
//    if (event->button() != Qt::LeftButton) {
//        return;
//    }
//    if(this->isMaximized()){
//        this->showNormal();
//        this->setFixedSize(900,620);
//        //主题库的全屏图标
//        m_title->btnMaximum->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
//        m_title->btnMaximum->setToolTip(tr("full srceen"));
//    }else{
//        this->setMaximumSize(16777215,16777215);
//        this->showMaximized();
//        //主题库的恢复图标
//        m_title->btnMaximum->setIcon(QIcon::fromTheme("window-restore-symbolic"));
//        m_title->btnMaximum->setToolTip(tr("recovery"));
//    }
//}

void horscreen::keyPressEvent(QKeyEvent *event)
{
    // F1快捷键打开用户手册
    if (event->key() == Qt::Key_F1) {
        if (!m_DaemonIpcDbus->daemonIsNotRunning()) {
            // F1快捷键打开用户手册，如kylin-photo-viewer
            //如果是小工具类，下面的showGuide参数要填写"tools/kylin-photo-viewer"
            m_DaemonIpcDbus->showGuide("kylin-weather");
        }
    }
}

// show或hide搜索弹窗
void horscreen::showSearchView()
{
    if (m_searchView->isHidden()) {
        m_searchView->show();
        emit searchClear();
        Interaction::getInstance()->loadCollectCity();
        m_city->m_changeCityBtn->setText(tr("【Retract】"));
    } else {
        m_searchView->hide();
        m_city->m_changeCityBtn->setText(tr("【Change】"));
    }
}

void horscreen::initInteraction()
{
    connect(Interaction::getInstance(), &Interaction::netUiShow, this,
            &horscreen::recieveNetSignal); //返回网络错误，给UI发信号
    connect(Interaction::getInstance(), &Interaction::serverUiShow, this,
            &horscreen::recieveNetSignal); //返回请求服务器错误，给UI发信号
    connect(Interaction::getInstance(), &Interaction::dataUiShow, this,
            &horscreen::recieveNetSignal); //返回数据解析错误，给UI发信号
    connect(Interaction::getInstance(), &Interaction::netTimeout, this, &horscreen::recieveNetSignal); //超时返回
    connect(Interaction::getInstance(), &Interaction::geoUiShow, this, &horscreen::geoUiShow);         //定位失败

    connect(Interaction::getInstance(), &Interaction::showCurrent, this, &horscreen::showCurrent); //接收当前城市信息
    connect(Interaction::getInstance(), &Interaction::showForcast, this, &horscreen::showForcast); //接收预测信息
    connect(Interaction::getInstance(), &Interaction::showLifeStyle, this,
            &horscreen::showLifeStyle); //接收生活指数信息
    connect(Interaction::getInstance(), &Interaction::showHourly, this,
            &horscreen::showHourly); //增加逐小时天气接口--临时
    connect(Interaction::getInstance(), &Interaction::closePro, this, &horscreen::closeActivated); //关闭程序-退出
}

void horscreen::initConnect()
{
    //最大化和还原
    //全屏按钮注释代码
    connect(m_title, &title::recovery, this, &horscreen::changOrigSize);
    connect(m_city->m_changeCityBtn, &QPushButton::clicked, this,
            &horscreen::showSearchView); //搜索-添加城市弹窗显隐，按钮text变化
    connect(m_searchView, &SearchView::searchWidState, this, &horscreen::showSearchView); //点击切换城市后的主界面处理
    connect(this, &horscreen::searchClear, m_searchView, &SearchView::searchReset); //搜索弹窗界面重置

    //当前城市信息
    connect(this, &horscreen::sendCurrentInfor, m_city, &city::getCurrentInfor);
    //当前城市天气对应的背景图
    connect(this, &horscreen::sendCurrentInfor, this, &horscreen::changeBackgroundImage);
    // lifestyle
    connect(this, &horscreen::sendLifeStyleInfor, m_information, &information::getLifeStyleInfor);
    // 发送信号到 information->onehourweather/onehourweather/lifestyle,判断是全屏还是普通界面
    connect(this, &horscreen::sendIsFullScreen, m_information, &information::sendIsFullScreen);
    //全屏后的 city 布局
    connect(this, &horscreen::sendIsFullScreen, m_city, &city::isFullScreen);
    //当前城市放大缩小
    connect(this, &horscreen::sendDetio, m_city, &city::initLayout);
    //当前城市上方空白widget放大缩小
    connect(this, &horscreen::sendDetio, m_empty, &Empty::resizeLayout);
    // 逐小时信息
    connect(this, &horscreen::sendHourlyInfor, m_information, &information::sendHourlyInfor);
    // 7周内天气信息
    connect(this, &horscreen::sendForcastInfor, m_information, &information::sendForcastInfor);
    // 7天预测天气图标放大缩小
    connect(this, &horscreen::sendWeatherIconInfor, m_information, &information::weekWeatherIcon);
    // 网络异常弹窗
    connect(this, &horscreen::recallInitLayout, m_hintWidget, &PromptWidget::initLayout);
    //    connect(m_city->m_changeCityBtn,&QPushButton::clicked,m_hintWidget,&PromptWidget::test);
    //    connect(this,&horscreen::recallInitLayout,m_hintWidget,&PromptWidget::test);
    connect(this, &horscreen::showCenterAfter, Interaction::getInstance(),
            &Interaction::showCenterAfter); //在屏幕中间显示

    connect(this, &horscreen::sendIsFullScreen, m_hintWidget, &PromptWidget::isFullScreen);

    connect(this, &horscreen::changeLoadingIconToButton, m_hintWidget, &PromptWidget::changeLoadingIconToButton);

    //    // 首次允许定位弹窗点击“是”更改菜单内允许定位为“是”
    //    connect(m_AllowLocationwidget,&AllowLocation::setMenuModuleAllowLocationYes,m_title,&title::setMenuModuleAllowLocationYes);
}

void horscreen::changOrigSize()
{
    if (this->isMaximized()) {
        this->showNormal();
        this->setFixedSize(900, 620);
        m_title->btnMaximum->setIcon(QIcon::fromTheme("window-maximize-symbolic")); //主题库的全屏图标
        m_title->btnMaximum->setToolTip(tr("full srceen"));
    } else {
        this->setMaximumSize(16777215, 16777215);
        this->showMaximized();
        m_title->btnMaximum->setIcon(QIcon::fromTheme("window-restore-symbolic")); //主题库的恢复图标
        m_title->btnMaximum->setToolTip(tr("recovery"));
    }
}

//不要删除
void horscreen::recieveNetSignal(QString netError)
{
    netErrorInfor = netError;
    //    showNetworkErrorDialog(netError);
    //    m_hintWidget->setVisible(true);
    //    m_hintWidget = new PromptWidget(this);
    //    m_hintWidget->setIconAndText(":/res/control_icons/network_warn.png", tr("Network not connected"));//网络未连接
    //    m_hintWidget->move((this->width() - m_hintWidget->width())/2, 70);
    //    m_hintWidget->setVisible(false);
    m_hintWidget->initLayout(netError, m_isFullScreen);
    m_mainCityLayout->removeWidget(m_city);
    m_city->hide();

    m_mainCityLayout->addWidget(m_empty, 0, Qt::AlignCenter);
    m_mainCityLayout->addWidget(m_hintWidget, 0, Qt::AlignCenter);

    //    m_hintWidget->move((this->width() - m_hintWidget->width())/2, 70);
    isShow = true;
    //    m_hintWidget->setVisible(true);
    m_empty->setVisible(isShow);
    m_hintWidget->setVisible(isShow);

    emit changeLoadingIconToButton();

    // 搜索弹窗出现后，网络出现异常，异常提示界面弹出，搜索弹窗应该收回
    m_searchView->hide();
}

void horscreen::geoUiShow(QString geoError)
{
    QDBusInterface iface("org.freedesktop.Notifications", "/org/freedesktop/Notifications",
                         "org.freedesktop.Notifications", QDBusConnection::sessionBus());
    QList<QVariant> args;
    args << (tr("kylin-weather")) << ((unsigned int)0)
         << QString("/usr/share/icons/ukui-icon-theme-classical/128x128/apps/kylin-weather.png")
         << tr("kylin-weather") //显示的是什么类型的信息  控制面板-更新提示
         << geoError + tr("About to switch to default City") //显示的具体信息
         << QStringList() << QVariantMap() << (int)-1;

    iface.callWithArgumentList(QDBus::AutoDetect, "Notify", args);
}

void horscreen::showCurrent(CurrentWeather m_currentWeather)
{
    m_city->show();
    isShow = false;
    m_empty->setVisible(isShow);
    m_hintWidget->setVisible(isShow);

    m_mainCityLayout->removeWidget(m_empty);
    m_mainCityLayout->removeWidget(m_hintWidget);

    m_mainCityLayout->addWidget(m_city, 0, Qt::AlignCenter);

    emit sendCurrentInfor(m_currentWeather);
    QString code = m_currentWeather.cond_code;
    //创建托盘图标
    //只有成功访问服务器且得到数据信息才显示托盘
    if (m_trayIcon != nullptr) {
        m_trayIcon->deleteLater();
    }
    createTrayIcon(code);

    m_label->resize(this->width(), this->height());
    QPixmap pixmap =
        m_pixmap.scaled(m_label->width(), m_label->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    m_label->setPixmap(pixmap);
    m_city->m_changeCityBtn->setEnabled(true);
}

void horscreen::showForcast(ForecastWeather forecastWeather)
{
    emit sendForcastInfor(forecastWeather, m_isFullScreen);
    // 判断前面是否有网导致界面显示出正常天气信息
    // 0 : 使用过网络； 1 : 未使用过(初始值为 1)
    // 确保 emit sendWeatherIconInfor(m_isFullScreen);只有在 horscreen::showForcast
    // 发送过 emit sendForcastInfor(forecastWeather,m_isFullScreen);后才能发送
    if (usedNet == 1) {
        usedNet = 0;
    }
    m_city->m_changeCityBtn->setEnabled(true);
}

void horscreen::showLifeStyle(LifeStyle m_lifeStyle)
{
    emit sendLifeStyleInfor(m_lifeStyle, m_isFullScreen);
}
//增加逐小时天气接口---临时用
void horscreen::showHourly(QVariant var)
{
    QList<Hourly> tmp_list = var.value<QList<Hourly>>();
    emit sendHourlyInfor(tmp_list, m_isFullScreen);
}
//创建托盘图标
void horscreen::createTrayIcon(QString code)
{
    m_trayIcon = new QSystemTrayIcon(this);
    m_trayIcon->setToolTip(QString(tr("Weather")));
    QString trayPath = ":/res/weather-icon/PNG16/2x/" + code + ".png";
    m_trayIcon->setIcon(QIcon(trayPath));
    //    m_trayIcon->setIcon()
    //添加托盘菜单
    m_mainMenu = new QMenu;
    m_openAction = new QAction(tr("Open Weather"), this); //打开麒麟天气
    m_quitAction = new QAction(tr("Exit"), this);         //退出
    m_mainMenu->addAction(m_openAction);
    m_openAction->setIcon(QIcon::fromTheme(QString("kylin-weather")));
    m_mainMenu->addAction(m_quitAction);
    m_quitAction->setIcon(QIcon::fromTheme(QString("exit-symbolic")));

    m_trayIcon->setContextMenu(m_mainMenu);
    m_trayIcon->show();

    connect(m_openAction, &QAction::triggered, this, [=] {
        if (this->isHidden() || this->isMinimized()) {
            handleIconClickedSub();
            Interaction::getInstance()->showCenterAfter();
            //            emit showCenterAfter();
        } else {
            return;
        }
    });
    //    connect(this,&horscreen::showInCenter,Interaction::getInstance,&Interaction::showInCenter);//在屏幕中间显示
    connect(m_quitAction, &QAction::triggered, this, &horscreen::closeActivated);
    connect(m_trayIcon, &QSystemTrayIcon::activated, this, &horscreen::iconActivated);
}

void horscreen::handleIconClickedSub()
{
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);

    //    if(this->m_menu && this->m_menu->aboutWindow){
    //        this->m_menu->aboutWindow->close();
    //    }
    if (this->m_title->g_menu->m_aboutWindow) {
        this->m_title->g_menu->m_aboutWindow->close();
    }
    if (!this->isVisible()) {
        QDesktopWidget *m = QApplication::desktop();
        QRect desk_rect = m->screenGeometry(m->screenNumber(QCursor::pos()));
        int desk_x = desk_rect.width();
        int desk_y = desk_rect.height();
        int x = this->width();
        int y = this->height();
        this->move(desk_x / 2 - x / 2 + desk_rect.left(), desk_y / 2 - y / 2 + desk_rect.top());
    }
    this->showNormal();
    this->raise();
    this->activateWindow();
    showLocateWidget();
}

//托盘图标被点击
void horscreen::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::MiddleClick:
        if (this->isHidden() || this->isMinimized()) {
            // this->showNormal();
            // handleIconClicked(); //靠近任务栏显示
            handleIconClickedSub(); //显示在屏幕中央
        } else {
            this->showMinimized();
        }
        break;
    case QSystemTrayIcon::DoubleClick:
        this->showMinimized();
        break;
    case QSystemTrayIcon::Context:
        //右键点击托盘图标弹出菜单
        // showTrayIconMenu(); //显示右键菜单
        m_mainMenu->show();
        break;
    default:
        break;
    }
}

void horscreen::closeActivated()
{
    //托盘退出默认关闭开机自启
    QString autostart =
        QStandardPaths::standardLocations(QStandardPaths::HomeLocation)[0] + "/.config/autostart/kylin-weather.desktop";
    QFile file(autostart);
    if (!file.exists()) {
        QString path = "/etc/xdg/autostart/kylin-weather.desktop";
        QFileInfo file2(path);
        if (!file2.exists()) {
            qInfo() << "/etc/xdg/autostart/目录下无麒麟天气快捷方式";
        } else {
            QFile::copy(path, autostart);
            if (file.exists()) {
                if (file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append)) {
                    file.write("\nHidden=true\n");
                    file.close();
                }
            }
        }
    }
    qApp->quit();
    return;
}

void horscreen::showLocateWidget()
{
    if (!(CoreVar::getSettings("locate-widget").toString() == "0" && !this->isHidden())) {
        return;
    }
    if (m_AllowLocationwidget == nullptr) {
        m_AllowLocationwidget = new AllowLocation(this);
        connect(m_AllowLocationwidget, &AllowLocation::setMenuModuleAllowLocationYes, m_title,
                &title::setMenuModuleAllowLocationYes);
        m_AllowLocationwidget->resize(350, 174);
    }
    //暂时规避方案
    QTimer *timer = new QTimer;
    timer->setSingleShot(true);
    connect(timer, &QTimer::timeout, this, &horscreen::showAllowLocationWidget);
    connect(timer, &QTimer::timeout, timer, &QTimer::deleteLater);
    timer->start(500);
}

void horscreen::showAllowLocationWidget()
{
    m_AllowLocationwidget->move(this->geometry().center() - m_AllowLocationwidget->rect().center());
    m_AllowLocationwidget->show();
}

//不要删--resize事件，使搜索弹窗可随主窗体尺寸变化而变化
void horscreen::resizeEvent(QResizeEvent *event)
{
    //    initLayout();
    if (m_isFullScreen == 1) {
        m_isFullScreen = 0;
    } else {
        m_isFullScreen = 1;
    }

    Q_UNUSED(event);
    m_searchView->move((this->width() - m_searchView->width()) / 2, (this->height() - m_searchView->height()) / 2);
    detio = double(this->height()) / 620;
    changeHeight(detio);
    m_label->resize(this->width(), this->height());
    QPixmap pixmap =
        m_pixmap.scaled(m_label->width(), m_label->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    m_label->setPixmap(pixmap);

    emit sendDetio(detio);
    emit sendIsFullScreen(m_isFullScreen);
    if (usedNet == 0) {
        //        qDebug()<<"horscreen::resizeEvent:usedNet:"<<usedNet;
        emit sendWeatherIconInfor(m_isFullScreen);
    }
    //    qDebug()<<"horscreen::resizeEvent:usedNet:"<<usedNet;
    // 如果网络提示界面是显示状态，则重新调用 PromptWidget::initLayout
    if (isShow == true) {
        m_hintWidget->initLayout(netErrorInfor, m_isFullScreen);
    }
    showLocateWidget();
}

//不要删--event事件，使搜索弹窗可在界面内失焦关闭
bool horscreen::event(QEvent *event)
{
    //点击别处，使搜索框失去焦点
    if (event->type() == QEvent::MouseButtonPress
        && !m_searchView->geometry().contains(this->mapFromGlobal(QCursor::pos()))) {
        //发送信号给搜索界面-重置界面显示
        emit searchClear();
        m_searchView->hide();
        m_city->m_changeCityBtn->setText(tr("【Change】"));
    }
    return QWidget::event(event);
}

void horscreen::hideSearchView()
{
    showSearchView();
}
