#include "onedayweather.h"
#include <QVBoxLayout>
#include <QLabel>
#include <QGraphicsOpacityEffect>
#include <QDebug>
#define UKUI_STYLE "org.ukui.style"
#define UKUI_FONT_SIZE "systemFontSize"
onedayweather::onedayweather(QWidget *parent) : QWidget(parent)
{
    m_onedayLayout = new QVBoxLayout(this);

    m_dayOfWeek = new QLabel(this);
    QString m_strDayOfWeek = "N/A";
    m_dayOfWeek->setText(m_strDayOfWeek);

    m_date = new QLabel(this);
    QString m_strDate = "N/A";
    m_date->setText(m_strDate);

    m_weatherIcon = new QLabel(this);
    QPixmap pixmap(":/res/forecast-icons/1x/300-2x.png");
    m_weatherIcon->setPixmap(pixmap);

    m_temp = new QLabel(this);
    QString m_strTemp = "N/A";
    m_temp->setText(m_strTemp);

    m_weather = new QLabel(this);
    QString m_strWeather = "N/A";
    m_weather->setText(m_strWeather);

    m_wind = new QLabel(this);
    QString m_strWind = "N/A";
    m_wind->setText(m_strWind);

    initFontSize();
    initLayout();
    //    initQss();
}

void onedayweather::initLayout()
{
    m_onedayLayout->addWidget(m_dayOfWeek);
    m_onedayLayout->addWidget(m_date);
    m_onedayLayout->addWidget(m_weatherIcon);
    m_onedayLayout->addWidget(m_temp);
    m_onedayLayout->addWidget(m_weather);
    m_onedayLayout->addWidget(m_wind);
    m_onedayLayout->setContentsMargins(0, 0, 0, 30);
}

void onedayweather::isFullScreen(int m_isFullScreen)
{
    m_judgeFullScreen = m_isFullScreen;
    initQss(14.0, 1.0);
}

void onedayweather::lable2SetFontSizeSlot(int size)
{
    //默认大小12px,换算成pt为9
    double lable2BaseFontSize = 11;                              //魔鬼数字，自行处理
    double nowFontSize = lable2BaseFontSize * double(size) / 11; // 11为系统默认大小，魔鬼数字，自行处理
    double times = double(size) / 11;
    //    QFont font;
    //    font.setPointSizeF(nowFontSize);
    //    ui->label2->setFont(font);
    initQss(nowFontSize, times);
}

void onedayweather::initFontSize()
{
    //只有非标准字号的控件才需要绑定
    connect(this, &onedayweather::setFontSizeSignal, this, &onedayweather::lable2SetFontSizeSlot);

    //判断gsettings是否存在
    if (QGSettings::isSchemaInstalled(UKUI_STYLE)) {
        m_fontSize = new QGSettings(UKUI_STYLE);
        //监听系统字号大小改变事件
        connect(m_fontSize, &QGSettings::changed, this, [=](const QString &key) {
            if (key == UKUI_FONT_SIZE) {
                //获取字号的值
                int fontSizeKey = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
                //发送改变信号
                if (fontSizeKey > 0) {
                    emit setFontSizeSignal(fontSizeKey);
                }
            }
        });
    }
    //启动时设置字号
    int fontSizeKey = 11; //系统默认字号,魔鬼数字，自行处理
    if (m_fontSize != nullptr) {
        int fontSizeKeyTmp = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
        if (fontSizeKeyTmp > 0) {
            fontSizeKey = fontSizeKeyTmp;
        }
    }
    emit setFontSizeSignal(fontSizeKey);
}

void onedayweather::initQss(double nowFontSize, double times)
{
    if (m_judgeFullScreen == 0) {
        m_font.setPointSize(int(11 * times));
        m_dayOfWeek->setFont(m_font);
        //    设置居中
        m_dayOfWeek->setAlignment(Qt::AlignCenter);
        //    设置显示白色字体
        QPalette dayOfWeekPalette = m_dayOfWeek->palette();
        dayOfWeekPalette.setColor(QPalette::WindowText, Qt::white);
        m_dayOfWeek->setPalette(dayOfWeekPalette);

        //    m_date->setStyleSheet("font:14px;color:rgba(255,255,255,0.7);");
        //    设置字体白色
        QPalette datePalette = m_date->palette();
        datePalette.setColor(QPalette::WindowText, Qt::white);
        m_date->setPalette(datePalette);
        //    设置字体透明度
        QGraphicsOpacityEffect *dateOpacityEffect = new QGraphicsOpacityEffect;
        dateOpacityEffect->setOpacity(0.7);
        m_date->setGraphicsEffect(dateOpacityEffect);
        //    设置居中
        m_date->setAlignment(Qt::AlignCenter);
        QFont m_dateAndTemp;
        //            m_font.setPointSizeF(nowFontSize);
        m_date->setFont(m_dateAndTemp);

        m_weatherIcon->setAlignment(Qt::AlignCenter);
        //            m_weatherIcon->resize(36,33);

        //    m_temp->setStyleSheet("font:14px;color:white;");
        QPalette tempPalette = m_temp->palette();
        tempPalette.setColor(QPalette::WindowText, Qt::white);
        m_temp->setPalette(tempPalette);
        m_temp->setAlignment(Qt::AlignCenter);
        m_temp->setFont(m_dateAndTemp);

        //    m_weather->setStyleSheet("font:12px;color:white;");
        //            m_font.setPixelSize(int(12 * times));
        m_font.setPointSize(int(9 * times));
        m_weather->setFont(m_font);
        QPalette weatherPalette = m_weather->palette();
        weatherPalette.setColor(QPalette::WindowText, Qt::white);
        m_weather->setPalette(weatherPalette);
        m_weather->setAlignment(Qt::AlignCenter);
        //    设置字体透明度
        QGraphicsOpacityEffect *weatherOpacityEffect = new QGraphicsOpacityEffect;
        weatherOpacityEffect->setOpacity(0.7);
        m_weather->setGraphicsEffect(weatherOpacityEffect);

        //    m_wind->setStyleSheet("font:12px;color:white;");
        m_font.setPointSize(int(9 * times));
        m_wind->setFont(m_font);
        QPalette windPalette = m_wind->palette();
        windPalette.setColor(QPalette::WindowText, Qt::white);
        m_wind->setPalette(windPalette);
        m_wind->setAlignment(Qt::AlignCenter);
        //    设置字体透明度
        QGraphicsOpacityEffect *windOpacityEffect = new QGraphicsOpacityEffect;
        windOpacityEffect->setOpacity(0.7);
        m_wind->setGraphicsEffect(windOpacityEffect);
    } else {
        m_font.setPixelSize(32);
        m_dayOfWeek->setFont(m_font);

        m_font.setPixelSize(28);
        m_date->setFont(m_font);

        m_temp->setFont(m_font);

        m_font.setPixelSize(24);
        m_weather->setFont(m_font);

        m_wind->setFont(m_font);
    }
}
