#include "onehourweather.h"
#include <QDebug>
#include <QThread>
#define UKUI_STYLE "org.ukui.style"
#define UKUI_FONT_SIZE "systemFontSize"
onehourweather::onehourweather(QWidget *parent) : QWidget(parent)
{
    m_onehourLayout = new QVBoxLayout(this);

    m_time = new QLabel(this);
    m_icon = new QLabel(this);
    m_temperature = new QLabel(this);

    m_time->setText("N/A");

    QPixmap m_iconPixmap(":/res/weather-icon/PNG16/2x/999.png");
    m_icon->setPixmap(m_iconPixmap);

    m_temperature->setText("N/A");

    initFontSize();
    initLayout();
}

void onehourweather::initLayout()
{
    m_onehourLayout->addWidget(m_time);
    m_onehourLayout->addWidget(m_icon);
    m_onehourLayout->addWidget(m_temperature);
}

void onehourweather::resizeEvent(QResizeEvent *event)
{
    initQss(0.0);
}

void onehourweather::isFullScreen(int m_isFullScreen)
{
    m_judgeFullScreen = m_isFullScreen;
    initQss(14.0);
}

void onehourweather::lable2SetFontSizeSlot(int size)
{
    //默认大小12px,换算成pt为9
    double lable2BaseFontSize = 11;//魔鬼数字，自行处理
    double nowFontSize = lable2BaseFontSize * double(size) / 11;//11为系统默认大小，魔鬼数字，自行处理
    initQss(nowFontSize);
}

void onehourweather::initFontSize()
{
    //只有非标准字号的控件才需要绑定
    connect(this,&onehourweather::setFontSizeSignal,this,&onehourweather::lable2SetFontSizeSlot);

    //判断gsettings是否存在
    if(QGSettings::isSchemaInstalled(UKUI_STYLE)) {
        m_fontSize = new QGSettings(UKUI_STYLE);
        //监听系统字号大小改变事件
        connect(m_fontSize,&QGSettings::changed,this, [=] (const QString &key) {
            if(key == UKUI_FONT_SIZE) {
                //获取字号的值
                int fontSizeKey = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
                //发送改变信号
                if (fontSizeKey > 0) {
                    emit setFontSizeSignal(fontSizeKey);
                }
            }
        });
    }
    //启动时设置字号
    int fontSizeKey = 11;//系统默认字号,魔鬼数字，自行处理
    if (m_fontSize != nullptr) {
        int fontSizeKeyTmp = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
        if (fontSizeKeyTmp > 0) {
            fontSizeKey = fontSizeKeyTmp;
        }
    }
    emit setFontSizeSignal(fontSizeKey);
}

void onehourweather::initQss(double nowFontSize)
{
//    m_icon->setAlignment(Qt::AlignCenter);

    if(m_judgeFullScreen == 0){
        QFont m_timeFont;
//        m_timeFont.setPointSizeF(nowFontSize);
        m_time->setFont(m_timeFont);
//        m_time->setStyleSheet("font:14px;color:white;");
        QPalette  timePalette = m_time->palette();
        timePalette.setColor(QPalette::WindowText,Qt::white);
        m_time->setPalette(timePalette);

//    m_icon->setStyleSheet("font:14px;color:white;");
//        QPalette  iconPalette = m_icon->palette();
//        iconPalette.setColor(QPalette::WindowText,Qt::white);
//        m_icon->setPalette(iconPalette);
//        m_icon->resize(36,33);
//        QPixmap m_iconPixmap(":/res/forecast-icons/1x/300-2x.png");
//        m_iconPixmap = m_iconPixmap.scaled(m_icon->width(), m_icon->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
//        m_icon->setPixmap(m_iconPixmap);

//    m_temperature->setStyleSheet("font:14px;color:white;");
        QPalette  tempPalette = m_temperature->palette();
        tempPalette.setColor(QPalette::WindowText,Qt::white);
        m_temperature->setPalette(tempPalette);
        m_temperature->setFont(m_timeFont);
        m_time->setAlignment(Qt::AlignCenter);
        m_temperature->setAlignment(Qt::AlignCenter);
    }
    else{
        QFont m_timeFont;
        m_timeFont.setPixelSize(28);
        m_time->setFont(m_timeFont);
        m_temperature->setFont(m_timeFont);
//        m_time->setStyleSheet("font:28px;color:white;");
//        m_icon->setStyleSheet("font:14px;color:white;");
//        m_icon->resize(72,66);
//        QPixmap m_iconPixmap(":/res/forecast-icons/1x/300-2x.png");
//        m_iconPixmap = m_iconPixmap.scaled(m_icon->width(), m_icon->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
//        m_icon->setPixmap(m_iconPixmap);
//        m_temperature->setStyleSheet("font:28px;color:white;");
        m_time->setAlignment(Qt::AlignCenter);
        m_temperature->setAlignment(Qt::AlignCenter);
    }
}

