#include "promptwidget.h"
#include "xatom-helper.h"
#include <QDebug>
#define UKUI_STYLE "org.ukui.style"
#define UKUI_FONT_SIZE "systemFontSize"
PromptWidget::PromptWidget(QWidget *parent) : QWidget(parent)
{
    this->setFixedSize(360,180);

    m_networkErrorLayout = new QHBoxLayout(this);

    m_networkWarnIconLabel = new QLabel;
    m_networkErrorIconLayout = new QHBoxLayout();

    m_networkErrorRightLayout = new QVBoxLayout();
    m_networkErrorCode = new QLabel;
    m_networkErrorCode->setAlignment(Qt::AlignCenter);

    m_networkErrorRightRetryButtonLayout = new QHBoxLayout();
    m_retryButton = new QPushButton();
    m_retryButton->setText(tr("Retry"));

    m_lodingIconLabel = new QLabel;
    m_lodingIconLabel->setVisible(false);
    m_lodingIconLabel->setAlignment(Qt::AlignCenter);
    m_lodingIcon = QPixmap(":/res/contorl_icons/loding_normal.png");
    m_lodingIconLabel->setPixmap(m_lodingIcon);

    addItems();
    initConnect();
    initFontSize();
}

void PromptWidget::initConnect()
{
    connect(m_retryButton,&QPushButton::clicked,this,&PromptWidget::changeButtonToLodingIcon);
    connect(m_retryButton,&QPushButton::clicked,Interaction::getInstance(),&Interaction::refresh);
    connect(ViewVar::getInstance(),&ViewVar::monitorThemeChanged,this,&PromptWidget::monitorThemeChanged);
}

void PromptWidget::changeButtonToLodingIcon()
{
    m_retryButton->setVisible(false);
    m_lodingIconLabel->setVisible(true);
}

void PromptWidget::changeLoadingIconToButton()
{
    m_retryButton->setVisible(true);
    m_lodingIconLabel->setVisible(false);
}

//可能存在全屏后图片无法跟随放大问题
void PromptWidget::monitorThemeChanged(QString themeChanged)
{
    if ("ukui-dark" == themeChanged || "ukui-black" == themeChanged) {
        m_iconPath = ":/res/contorl_icons/network_warn_dark.png";
        m_lodingIconPath = ":/res/contorl_icons/loding_dark.png";
    } else {
        m_iconPath = ":/res/contorl_icons/network_warn_normal.png";
        m_lodingIconPath = ":/res/contorl_icons/loding_normal.png";
    }

    if (m_judgeFullScreen == 0)
    {
        m_networkWarnIconLabel->resize(77,77);
    }
    else
    {
        m_networkWarnIconLabel->resize(154,154);
    }
    m_networkErrorIcon = QPixmap(m_iconPath);
    m_networkErrorIcon = m_networkErrorIcon.scaled(m_networkWarnIconLabel->width(), m_networkWarnIconLabel->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    m_networkWarnIconLabel->setPixmap(m_networkErrorIcon);

    m_lodingIcon = QPixmap(m_lodingIconPath);
    m_lodingIconLabel->setPixmap(m_lodingIcon);
}

void PromptWidget::isFullScreen(int m_isFullScreen)
{
    m_judgeFullScreen = m_isFullScreen;
    initLayout(netErrorWords , m_isFullScreen);
}

void PromptWidget::lable2SetFontSizeSlot(int size)
{
    initLayout(netErrorWords , m_judgeFullScreen);
}

void PromptWidget::initFontSize()
{
    //只有非标准字号的控件才需要绑定
    connect(this,&PromptWidget::setFontSizeSignal,this,&PromptWidget::lable2SetFontSizeSlot);

    //判断gsettings是否存在
    if(QGSettings::isSchemaInstalled(UKUI_STYLE)) {
        m_fontSize = new QGSettings(UKUI_STYLE);
        //监听系统字号大小改变事件
        connect(m_fontSize,&QGSettings::changed,this, [=] (const QString &key) {
            if(key == UKUI_FONT_SIZE) {
                //获取字号的值
                int fontSizeKey = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
                //发送改变信号
                if (fontSizeKey > 0) {
                    emit setFontSizeSignal(fontSizeKey);
                }
            }
        });
    }
    //启动时设置字号
    int fontSizeKey = 11;//系统默认字号,魔鬼数字，自行处理
    if (m_fontSize != nullptr) {
        int fontSizeKeyTmp = m_fontSize->get(UKUI_FONT_SIZE).toString().toInt();
        if (fontSizeKeyTmp > 0) {
            fontSizeKey = fontSizeKeyTmp;
        }
    }
    emit setFontSizeSignal(fontSizeKey);
}

void PromptWidget::initLayout(QString netError,int m_isFullScreen)
{
    netErrorWords = netError;
    // 普通界面大小
    if(m_isFullScreen == 0)
    {
        this->setFixedSize(360,140);
        m_networkWarnIconLabel->resize(77,77);
//        m_iconPath = ":/res/contorl_icons/network_warn_normal.png";
        m_networkErrorIcon = QPixmap(m_iconPath);
        m_networkErrorIcon = m_networkErrorIcon.scaled(m_networkWarnIconLabel->width(), m_networkWarnIconLabel->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
        m_networkWarnIconLabel->setPixmap(m_networkErrorIcon);

        m_networkErrorCode->setText(netError);
        m_networkErrorCode->setAlignment(Qt::AlignVCenter);
        m_networkErrorCode->resize(122,20);
        // 设置字号
        QFont networkErrorCodeFont;
//        m_font.setPixelSize(14);
        m_networkErrorCode->setFont(networkErrorCodeFont);

        m_retryButton->setFixedSize(96,35);
        m_retryButton->setFont(networkErrorCodeFont);

        m_lodingIconLabel->setFixedSize(96,35);
//        m_retryButtonIconPath = ":/res/contorl_icons/retry_normal.png";
//        m_retryButtonIcon = QPixmap(m_retryButtonIconPath);
//        m_retryButtonIcon = m_retryButtonIcon.scaled(m_retryButton->width(), m_retryButton->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
//        m_retryButton->setIcon(m_retryButtonIcon);
//        m_retryButton->setIconSize(QSize(m_retryButton->width(),m_retryButton->height()));
//        m_retryButton->setFlat(true);//按钮背景为透明，但是有边框
    }
    //全屏界面大小
    else
    {
        this->setFixedSize(632,280);
        m_networkWarnIconLabel->resize(154,154);
//        m_iconPath = ":/res/contorl_icons/network_warn_normal.png";
        m_networkErrorIcon = QPixmap(m_iconPath);
        m_networkErrorIcon = m_networkErrorIcon.scaled(m_networkWarnIconLabel->width(), m_networkWarnIconLabel->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
        m_networkWarnIconLabel->setPixmap(m_networkErrorIcon);

        m_networkErrorCode->setText(netError);
        m_networkErrorCode->setAlignment(Qt::AlignVCenter);
        m_networkErrorCode->resize(280,28);
        // 设置字号
        m_font.setPixelSize(32);
        m_networkErrorCode->setFont(m_font);

        m_retryButton->setFixedSize(202,70);
        m_retryButton->setFont(m_font);

        m_lodingIconLabel->setFixedSize(202,70);
//        m_retryButtonIconPath = ":/res/contorl_icons/retry_normal.png";
//        m_retryButtonIcon = QPixmap(m_retryButtonIconPath);
//        m_retryButtonIcon = m_retryButtonIcon.scaled(m_retryButton->width(), m_retryButton->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
//        m_retryButton->setIcon(m_retryButtonIcon);
//        m_retryButton->setIconSize(QSize(m_retryButton->width(),m_retryButton->height()));
//        m_retryButton->setFlat(true);//按钮背景为透明，但是有边框
    //    QHBoxLayout *m_networkErrorRightRetryButtonLayout = new QHBoxLayout();
    }
}

void PromptWidget::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QPainter painter(this);
    QStyleOption opt;
    opt.init(this);
    painter.setRenderHint(QPainter::Antialiasing); // 反锯齿;
    painter.setBrush(QBrush(opt.palette.color(QPalette::Base)));
    painter.setPen(Qt::transparent);
    QRect rect = this->rect();
    rect.setWidth(rect.width() );
    rect.setHeight(rect.height() );
    painter.drawRoundedRect(rect, 6, 6);
}

void PromptWidget::addItems()
{
    m_networkErrorLayout->addStretch(2);

    m_networkErrorLayout->addWidget(m_networkWarnIconLabel);
    m_networkErrorLayout->addStretch(2);
    m_networkErrorRightLayout->addWidget(m_networkErrorCode);
    m_networkErrorRightRetryButtonLayout->addWidget(m_retryButton);
    m_networkErrorRightRetryButtonLayout->addWidget(m_lodingIconLabel);
    m_networkErrorRightLayout->addLayout(m_networkErrorRightRetryButtonLayout);
    m_networkErrorRightRetryButtonLayout->setContentsMargins(0,0,0,25);

    m_networkErrorLayout->addLayout(m_networkErrorRightLayout);
    m_networkErrorLayout->addStretch(2);
    this->setLayout(m_networkErrorLayout);
}
