#include "collectcity.h"

CollectCity::CollectCity(QWidget *parent) : QWidget(parent)
{
    initInteraction();//信号槽--后端信号
    this->setFixedSize(ViewVar::COLLECT);
    this->setAttribute(Qt::WA_TranslucentBackground);//设置背景透明

    m_cityTitle = new QLabel(this);//“收藏城市”
    m_cityTitle->setText(tr("Collect City"));
    m_cityTitle->setFixedWidth(this->width());
    m_cityTitle->move(ViewVar::COLLECT_TITLE);

    m_noneCity = new QLabel(this);//“暂无”
    m_noneCity->setText(tr("None"));
    m_noneCity->setFixedWidth(this->width());
    m_noneCity->move(ViewVar::NONECITYX,m_cityTitle->y() + m_cityTitle->height() + 8);

    m_cityList = new QTableView(this);//收藏城市列表
    m_cityList->setFixedSize(ViewVar::COLLECT_LIST);
    m_cityList->move(ViewVar::CITYLISTX,m_cityTitle->y() + m_cityTitle->height() + 4);
    m_cityList->hide();//默认隐藏--打开时判断citylist有没有城市，再去设置显隐

    m_cityList->setAlternatingRowColors(false);//设置无相邻行颜色区分
    m_cityList->setShowGrid(false);//设置无网格线
    m_cityList->setEditTriggers(QAbstractItemView::NoEditTriggers);//设置不可编辑
    //设置行高列宽
    m_cityList->verticalHeader()->setDefaultSectionSize(ViewVar::COLLECT_ITEMH);
    m_cityList->horizontalHeader()->setDefaultSectionSize(ViewVar::COLLECT_ITEMW);
    //设置代理
    m_delegate = new CollectItemDelegate(this);
    m_cityList->setItemDelegate(m_delegate);
    m_cityList->horizontalHeader()->setVisible(false);// 水平不可见
    m_cityList->verticalHeader()->setVisible(false);// 垂直不可见
    m_cityList->setSelectionMode(QAbstractItemView::NoSelection);//设置不可多选
    m_cityList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);//设置无水平滑动条
    m_cityList->verticalScrollBar()->setProperty("drawScrollBarGroove",false);//设置滑动条无滑槽
    m_cityList->setFrameShape(QFrame::NoFrame);//设置无边框
    m_cityList->setAttribute(Qt::WA_TranslucentBackground);//设置背景透明
    QPalette scrollarea = m_cityList->palette();
    scrollarea.setColor(QPalette::Base, ViewVar::s_colorWid);
    m_cityList->setPalette(scrollarea);
    setModelforView();
    initConnect();
}
//设置model
void CollectCity::setModelforView()
{
    m_cityList->setModel(Interaction::getInstance()->getCollectModel());
}
//信号槽--后端信号
void CollectCity::initInteraction()
{
    connect(Interaction::getInstance(),&Interaction::collectNull,this,&CollectCity::changeItem);//判断收藏城市为空时，交换两个控件的显隐
    connect(Interaction::getInstance(),&Interaction::collectShow,this,&CollectCity::changeShow);//判断不为空，两个控件的显隐状态
}
//信号槽
void CollectCity::initConnect()
{
    connect(m_delegate,&CollectItemDelegate::delCollectCity,Interaction::getInstance(),&Interaction::delCollectCity);//删除收藏列表城市
    connect(m_delegate,&CollectItemDelegate::changeCityFromCollect,this,&CollectCity::changeCityFromCollect);//从收藏城市改变当前城市
}
//收藏城市为空，收藏列表隐藏
void CollectCity::changeItem()
{
    if (m_cityList->isHidden()) {
        return;
    }
    m_cityList->hide();
    m_noneCity->show();
    emit changeItemPos();
}
//判断不为空，两个控件的显隐状态
void CollectCity::changeShow()
{
    if (!m_cityList->isHidden()) {
        return;
    }
    m_cityList->show();
    m_noneCity->hide();
    emit changeItemPos();
}
