#include "searchview.h"
SearchView::SearchView(QWidget *parent) : QWidget(parent)
{
    initInteraction();
    this->setFixedSize(ViewVar::SEARCH);
    m_searchBox = new SearchBox(this);
    m_searchBox->setFixedSize(ViewVar::SEARCH_BOX);
    m_searchBox->move((this->width()-m_searchBox->width())/2,ViewVar::SEARCH_BOXY);

    m_hotCity = new HotCity(this);
    m_collectCity = new CollectCity(this);
    m_searchCityList = new SearchCityLIst(this);
    m_searchCityList->hide();

    this->changePostion();
    this->initConnect();

}

void SearchView::initInteraction()
{
    connect(Interaction::getInstance(),&Interaction::searchUiShow,this,&SearchView::searchListUi);
}

void SearchView::initConnect()
{
    connect(m_hotCity,&HotCity::setHotCity,this,&SearchView::changeCity);//点击热门城市按钮，改变当前城市
    connect(m_searchBox,&SearchBox::textChanged,this,&SearchView::sendSearchInput);//处理输入字符串，处理后返回给后端
    connect(this,&SearchView::shouldRestore,m_searchBox,&SearchBox::restoreStatus);//处理搜索框的显示
    connect(m_searchCityList,&SearchCityLIst::addCollectCity,this,&SearchView::addCollectCity);//添加收藏城市
    connect(m_searchCityList,&SearchCityLIst::changeCityFromSearch,this,&SearchView::changeCityFromSearch);//从搜索界面切换城市
    connect(m_collectCity,&CollectCity::changeCityFromCollect,this,&SearchView::changeCityFromCollect);//从收藏城市列表改变当前城市
    connect(m_collectCity,&CollectCity::changeItemPos,this,&SearchView::changePostion);//当收藏列表为空时，改变位置
    connect(m_searchCityList,&SearchCityLIst::cityListNull,this,&SearchView::searchListUi);//处理搜索为空界面显示
}
//切换城市--界面处理
void SearchView::changeUiShow()
{
    searchReset();
    emit searchWidState();
}
//搜索-收藏隐藏--界面处理
void SearchView::searchReset()
{
    if (m_searchCityList->isHidden()) {
        return;
    }
    m_searchCityList->hide();
    m_collectCity->show();
    m_hotCity->show();
    m_searchBox->dealUiShow();
}
//搜索界面各部分显隐和位置确定
void SearchView::searchListUi(bool hasCity)
{
    if (hasCity) {
        m_hotCity->hide();
        m_collectCity->hide();
        m_searchCityList->show();
        emit shouldRestore(false);
        m_searchCityList->move(ViewVar::SEARCHLIST_POS);
    } else {
        m_hotCity->show();
        m_collectCity->show();
        m_searchCityList->hide();
        emit shouldRestore(true);
    }
    m_searchCityList->searchUiChange(hasCity);
}
void SearchView::sendSearchInput()
{
    QString inputText = m_searchBox->text().trimmed().toLower();
    Interaction::getInstance()->searchResult(inputText);
}
//添加收藏城市
void SearchView::addCollectCity(QModelIndex index)
{
    Interaction::getInstance()->addCollectCity(index);
    m_searchCityList->hide();
    m_hotCity->show();
    m_collectCity->show();
    if (m_collectCity->m_cityList->isHidden()) {
        m_collectCity->m_cityList->show();
        m_collectCity->m_noneCity->hide();
        changePostion();
    }
    m_searchBox->dealUiShow();
}
//切换主界面城市-从搜索界面
void SearchView::changeCityFromSearch(QModelIndex index)
{
    emit searchWidState();
    Interaction::getInstance()->changeCityFromSearch(index);

}
//切换主界面城市-从收藏界面
void SearchView::changeCityFromCollect(QModelIndex index)
{
    changeUiShow();
    Interaction::getInstance()->changeCityFromCollect(index);
}
//确定热门城市和收藏城市位置
void SearchView::changePostion()
{
    //如果收藏城市无城市，则将热门城市放在上边
    if (!m_collectCity->m_noneCity->isHidden()) {
        m_hotCity->move(ViewVar::SEARCH_CHANGECITYX,ViewVar::DISTANCE1 + m_searchBox->y() + m_searchBox->height());
        m_collectCity->move(m_hotCity->x(),m_hotCity->y()+m_hotCity->height() + ViewVar::DISTANCE0);
    } else {
        m_collectCity->move(ViewVar::SEARCH_CHANGECITYX,ViewVar::DISTANCE1 + m_searchBox->y() + m_searchBox->height());
        m_hotCity->move(m_collectCity->x(),m_collectCity->y()+m_collectCity->height() + ViewVar::DISTANCE0);
    }
}
//画圆角
void SearchView::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QPainter painter(this);
    QStyleOption opt;
    opt.init(this);
    painter.setRenderHint(QPainter::Antialiasing); // 反锯齿;
    //在全局模块设置的变量在这里使用时不生效，还没有找到原因，先直接使用数字解决bug，之后再解决
    if (ViewVar::s_themeStyle) {
//        painter.setBrush(QBrush(QColor(246,246,246)));
        painter.setBrush(QBrush(QColor(255,255,255)));
    } else {
        painter.setBrush(QBrush(QColor(0,0,0)));
    }
    painter.setPen(Qt::transparent);
    QRect rect = this->rect();
    rect.setWidth(rect.width() );
    rect.setHeight(rect.height() );
    painter.drawRoundedRect(rect, 6, 6);

}

bool SearchView::event(QEvent *event)
{
    //点击别处，使搜索框失去焦点
    if (event->type() == QEvent::MouseButtonPress)
    {
        m_searchBox->clearFocus();
    }
    return QWidget::event(event);
}
//切换城市--从热门城市
void SearchView::changeCity(QString cityId)
{
    changeUiShow();
    Interaction::getInstance()->changeCity(cityId);
}

